require 'spec_helper'

describe Puppet::Type.type(:evs_properties) do
  let(:required_params) do
    {
      :name   => 'controller_property',
      :ensure => 'present'
    }
  end
  let(:optional_params) do
    {
      # properties
        # :l2_type => ,
        # :uplink_port => ,
        # :uri_template => ,
        # :vlan_range => ,
        # :vxlan_addr => ,
        # :vxlan_ipvers => ,
        # :vxlan_mgroup => ,
        # :vxlan_range => ,
        # :controller => ,
      # parameters
    }
  end

  # Modify the params inline to tests when you modeling the
  # behavior of the generated resource
  let(:params) { required_params.merge(optional_params) }
  let(:resource) { described_class.new(params) }
  let(:provider) { Puppet::Provider.new(resource) }
  let(:catalog) { Puppet::Resource::Catalog.new }
  let(:error_pattern) { %r(failed on) }

  it "has :name as its keyattribute" do
    expect( described_class.key_attributes).to eq([:name])
  end

  describe "has property" do
    [
      # list properties
        :l2_type,
        :uplink_port,
        :uri_template,
        :vlan_range,
        :vxlan_addr,
        :vxlan_ipvers,
        :vxlan_mgroup,
        :vxlan_range,
        :controller,
    ].each { |prop|
      it prop do
        expect(described_class.attrtype(prop)).to eq(:property)
      end
    }
  end
  describe "has parameter" do
    [
        :name,
    ].each { |param|
      it param do
        expect(described_class.attrtype(param)).to eq(:param)
      end
    }
  end

# Input Validation
describe "parameter" do
  [:name].each do |type|
    context "#{type}" do
      context "accepts" do
        %w(controller_property client_property).each do |thing|
          it thing.inspect do
            params[type] = thing
            expect { resource }.not_to raise_error
          end
        end
      end
      context "rejects" do
        %w(random).each do |thing|
          it thing.inspect do
            params[type] = thing
            expect { resource }.to raise_error(Puppet::Error, error_pattern)
          end
        end
      end
      it "fails if unset" do
        params.delete(type)
        expect { resource }.to raise_error(Puppet::Error, %r(must be provided))
      end
    end
  end
end # parameters

describe "property" do
  [:l2_type].each do |type|
    context "#{type}" do
      context "accepts" do
        %w(vlan vxlan flat).each do |thing|
          it thing.inspect do
            params[type] = thing
            expect { resource }.not_to raise_error
          end
        end
      end
      context "rejects" do
        %w(star).each do |thing|
          it thing.inspect do
            params[type] = thing
            expect { resource }.to raise_error(Puppet::Error, error_pattern)
          end
        end
      end
      it "does not fail if unset" do
        params.delete(type)
        expect { resource }.not_to raise_error
      end
      it "defaults to vlan" do
        expect(resource[type]).to eq(:vlan)
      end
    end
  end
    [:uplink_port].each do |type|
      context "#{type}" do
        context "accepts" do
          [
            *%w(
            net1
            net1;;;;
            net1;10-20;;;
            net1;10-20;10000-10100;;
            net1;10-20;10000-10100;foo;
            net1;10-20;10000-10100;foo;yes
            net1;10-20,45-50;10000-10100,10200-10250;foo.local;yes
            net1;;10000-10100,10200-10250;foo.local;yes
            ),
            {
              'uplink-port'  => 'net1',
              'vlan-range'   => '10-20',
              'vxlan-range'  => '10000-10100',
              'host'         => 'foo',
              'flat'         => 'yes'
            },
            {
              'uplink-port'  => 'net1',
              'vlan-range'   => '10-20,30-40',
              'vxlan-range'  => '10000-10100,10200-10250',
              'host'         => 'foo.local',
              'flat'         => 'yes'
            },
            [ # Array of hashes
              {
                'uplink-port'  => 'net1',
                'vlan-range'   => '10-20',
                'vxlan-range'  => '10000-10100',
                'host'         => 'foo',
                'flat'         => 'yes'
              },
              {
                'uplink-port'  => 'net1',
                'vlan-range'   => '10-20,30-40',
                'vxlan-range'  => '10000-10100,10200-10250',
                'host'         => 'foo.local',
                'flat'         => 'yes'
              },
            ]
          ].each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          [
            *%w(
            net1/v4
            net1;10-20
            net1;10-20,45-50;10000-10100;10200-10250;foo;yes
            net1;10-20,45-50;10000-10100,10200-10250;foo;yes;more
            ),
            {
              'uplink-port'  => 'net1/v4',
            },
            {
              'uplink-port'  => 'net1',
              'vlan-range'   => '10-20a',
            },
            {
              'uplink-port'  => 'net1',
              'vlan-range'   => '10-20,30-40a',
            },
            {
              'uplink-port'  => 'net1',
              'vxlan-range'  => '10000-10100a',
            },
            {
              'uplink-port'  => 'net1',
              'host'         => 'foo/bar',
            },
            {
              'uplink-port'  => 'net1',
              'flat'         => 'maybe'
            },
            [[]] # End result is a single empty array
          ].each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        context "munges" do
          %w(net1).each do |thing|
            it "simple string #{thing}" do
              params[type] = thing
              expect(resource[type]).to eq(%W(-p uplink-port=#{thing}))
            end
          end
          %w(net1;10-20;;;).each do |thing|
            it "string partial #{thing}" do
              params[type] = thing
              expect(resource[type]).to eq(["-p", "uplink-port=net1,vlan-range=10-20"])
            end
          end
          %w(net1;10-20;;;yes).each do |thing|
            it "string partial #{thing}" do
              params[type] = thing
              expect(resource[type]).to eq(["-p", "uplink-port=net1,vlan-range=10-20,flat=yes"])
            end
          end
          %w(net1;10-20;10000-10100;;).each do |thing|
            it "string partial #{thing}" do
              params[type] = thing
              expect(resource[type]).to eq(["-p", "uplink-port=net1,vlan-range=10-20,vxlan-range=10000-10100"])
            end
          end
          %w(net1;10-20;10000-10100;foo;).each do |thing|
            it "string partial #{thing}" do
              params[type] = thing
              expect(resource[type]).to eq(["-h", "foo", "-p", "uplink-port=net1,vlan-range=10-20,vxlan-range=10000-10100"])
            end
          end
          %w(net1;10-20;10000-10100;foo;yes).each do |thing|
            it "string partial #{thing}" do
              params[type] = thing
              expect(resource[type]).to eq(["-h", "foo", "-p", "uplink-port=net1,vlan-range=10-20,vxlan-range=10000-10100,flat=yes"])
            end
          end
          %w(net1;10-20,45-50;10000-10100,10200-10250;foo.local;yes)
          .each do |thing|
            it "string #{thing}" do
              params[type] = thing
              expect(resource[type]).to eq(["-h", "foo.local", "-p",
                "uplink-port=net1,vlan-range=10-20,45-50,vxlan-range=10000-10100,10200-10250,flat=yes"])
              end
            end
            [
              {
                'uplink-port'  => 'net1',
                'vlan-range'   => '10-20,45-50',
                'vxlan-range'  => '10000-10100,10200-10250',
                'host'         => 'foo.local',
                'flat'         => 'yes'
              }
            ].each do |thing|
              it "hash #{thing}" do
                params[type] = thing
                expect(resource[type]).to eq(["-h", "foo.local", "-p",
                  "uplink-port=net1,vlan-range=10-20,45-50,vxlan-range=10000-10100,10200-10250,flat=yes"])
                end
              end
              [
                {
                  'uplink-port'  => 'net1',
                  'vlan-range'   => '10-20,30-40',
                  'host'         => 'foo.local',
                  'flat'         => 'yes'
                }
              ].each do |thing|
                it "partial hash #{thing}" do
                  params[type] = thing
                  expect(resource[type]).to eq(["-h", "foo.local", "-p",
                    "uplink-port=net1,vlan-range=10-20,30-40,flat=yes"])
                  end
                end
              end
            end
    end

    [:uri_template].each do |type|
      context "#{type}" do
        context "accepts" do
          (%w(
            unix://
            unix://controller
            unix://;controller
            ssh://
            ssh://controller
            ssh://;controller
            absent
          ) + [%w(unix:// ssh://)]).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "munges" do
          [
          [ %q(unix://), %w(-p unix://)],
          [ %q(unix://controller), %w(-h controller -p unix://)],
          [ %q(unix://;controller), %w(-h controller -p unix://)],
          [ %q(ssh://), %w(-p ssh://)],
          [ %q(ssh://controller), %w(-h controller -p ssh://)],
          [ %q(ssh://;controller), %w(-h controller -p ssh://)],
        ].each do |thing|
            it thing[0].inspect do
              params[type] = thing[0]
              thing[1][-1].insert(0,"uri-template=")
              expect(resource[type]).to eq([thing[1]])
            end
          end
          it %w(unix:// ssh://).inspect do
            params[type] = %w(unix:// ssh://)
            expect(resource[type]).to eq(
            [["-p", "uri-template=unix://"], ["-p", "uri-template=ssh://"]])
          end
          it %w(unix://foo ssh://).inspect do
            params[type] = %w(unix://foo ssh://)
            expect(resource[type]).to eq(
            [["-h", "foo", "-p", "uri-template=unix://"], ["-p", "uri-template=ssh://"]])
          end
        end
        context "rejects" do
          %w(
            unix://foo/
            ssh://bar?
            https://anything
          ).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
       it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:vlan_range].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(1-10 10-50,90-4094).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(10-1 0-5 4000-5000 1-10,35-20).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
       it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:vxlan_addr].each do |type|
      context "#{type}" do
        context "accepts" do
          (
          ['1.2.3.4', 'fe80::3e07:54ff:fe53:c704'] +
          ['1.2.3.4', 'fe80::3e07:54ff:fe53:c704'].
            product(['/24',nil]).map(&:join).
            product([nil,'100-300']).
            product([nil,'foo']).
            map{|a| a.join(';')}
          ).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        ## Test any munge behavior for parameter here
        xit "munges" do
          params[type] = %w(random_string)
          expect(resource[type]).to eq('other_string')
        end
        context "rejects" do
          %w(1.2.3.4/234 1.2.3.456 Pe80::3e07:54ff:fe53:c704).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
       it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:vxlan_ipvers].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(v6 v4).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(IPv4).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
       it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        it "defaults to v4" do
          expect(resource[type]).to eq(:v4)
        end
      end
    end
    [:vxlan_mgroup].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(224.0.0.1 FF00::1).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(1.2.3.4 FE00::).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:vxlan_range].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(1-10 0-16777215 1-10,20-30).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(-1 1.1 16777216 30-20 1-20,40-30).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
       it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        ## Test any munge behavior for parameter here
        #it "munges" do
        #  params[type] = %w(random_string)
        #  expect(resource[type]).to eq('other_string')
        #end
      end
    end
    [:controller].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(unix:// ssh://foo.local ssh://user@foo.local).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(ftp://foo ssh://@foo.local).each do |thing|
           it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
       it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
       xit "fails with name == 'controller_property'"
        ## Test any munge behavior for parameter here
        #it "munges" do
        #  params[type] = %w(random_string)
        #  expect(resource[type]).to eq('other_string')
        #end
      end
    end
end # properties


# describe "autorequire" do
#   # This assumes resource requires thing => two
#   it "does not require thing when no matching resource exists" do
#     thing = Puppet::Type.type(:thing).new(:name => "one")
#     catalog.add_resource thing
#     catalog.add_resource resource
#     expect(resource.autorequire.count).to eq 0
#   end
#   it "requires the matching thing resource" do
#     thing = Puppet::Type.type(:thing).new(:name => "two")
#     catalog.add_resource thing
#     catalog.add_resource resource
#     reqs = resource.autorequire
#     expect(reqs.count).to eq 1
#     expect(reqs[0].source).to eq thing
#     expect(reqs[0].target).to eq resource
#   end
# end
end
