require 'spec_helper'

describe Puppet::Type.type(:evs) do
  let(:required_params) do
    {
      :name   => 'tenant/switch',
      :ensure => 'present'
    }
  end
  let(:optional_params) do
    {
      # properties
        # :maxbw => ,
        # :priority => ,
        # :protection => ,
        # :l2_type => ,
        # :vlanid => ,
        # :vni => ,
        # :uuid => ,
      # parameters
    }
  end

  # Modify the params inline to tests when you modeling the
  # behavior of the generated resource
  let(:params) { required_params.merge(optional_params) }
  let(:resource) { described_class.new(params) }
  let(:provider) { Puppet::Provider.new(resource) }
  let(:catalog) { Puppet::Resource::Catalog.new }
  let(:error_pattern) { %r(failed on) }

  it "has :name as its keyattribute" do
    expect( described_class.key_attributes).to eq([:name])
  end

  describe "has property" do
    [
      # list properties
        :ensure,
        :maxbw,
        :priority,
        :protection,
        :l2_type,
        :vlanid,
        :vni,
        :uuid,
    ].each { |prop|
      it prop do
        expect(described_class.attrtype(prop)).to eq(:property)
      end
    }
  end
  describe "has parameter" do
    [
        :name,
    ].each { |param|
      it param do
        expect(described_class.attrtype(param)).to eq(:param)
      end
    }
  end

# Input Validation
describe "parameter" do
  [:name].each do |type|
    context "#{type}" do
      context "accepts" do
        %w(foo/bar).each do |thing|
          it thing.inspect do
            params[type] = thing
            expect { resource }.not_to raise_error
          end
        end
      end
      context "rejects" do
        %w(foo foo/bar/baz).each do |thing|
          it thing.inspect do
            params[type] = thing
            expect { resource }.to raise_error(Puppet::Error, error_pattern)
          end
        end
      end
      it "fails if unset" do
        params.delete(type)
        expect { resource }.to raise_error(Puppet::Error, %r(must be provided))
      end
    end
  end
end # parameters

describe "property" do
    [:ensure].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(present absent).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(yes no).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:maxbw].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(1 1m 1M 1k 1K 1g 1G).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(1T 1.1 string).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:priority].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(high medium low).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(fast slow).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        it "defaults to medium" do
          expect(resource[type]).to eq(:medium)
        end
      end
    end
    [:protection].each do |type|
      context "#{type}" do
        context "accepts" do
          # Build all the combinations
          (
            (1..4).flat_map{|size|
            %w(mac-nospoof restricted ip-nospoof dhcp-nospoof).
              combination(size).to_a } + [["none"]]
          ).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          (%w(other) + [[ "mac-nospoof", "none"]]).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error,
                %r(Validation.*failed|Invalid value))
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        it "defaults to mac-nospoof,ip-nospoof" do
          expect(resource[type]).to eq([:"mac-nospoof", :"ip-nospoof"])
        end
      end
    end
    [:l2_type].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(vlan vxlan flat).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(star).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        it "defaults to vlan" do
          expect(resource[type]).to eq(:vlan)
        end
      end
    end
    [:vlanid].each do |type|
      context "#{type}" do
        context "accepts" do
          [1, 100, 4094].each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          [0, -1, 1.1, 4095, "four"].each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:vni].each do |type|
      context "#{type}" do
        context "accepts" do
          [0, 10000, 16777215].each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          [-1, 1.1, 16777216, "four"].each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:uuid].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(6c982de8-df63-4d1b-b3af-441cf2eb1959).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(Pc982de8-df63-4d1b-b3af-441cf2eb1959).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
  end # properties
end
