require 'spec_helper'

describe Puppet::Type.type(:evs_vport) do
  let(:required_params) do
    {
      :name   => 'tenant/switch/port',
      :ensure => 'present'
    }
  end
  let(:optional_params) do
    {
      # properties
        # :cos => ,
        # :maxbw => ,
        # :priority => ,
        # :protection => ,
        # :ipaddr => ,
        # :macaddr => ,
        # :uuid => ,
      # parameters
    }
  end

  # Modify the params inline to tests when you modeling the
  # behavior of the generated resource
  let(:params) { required_params.merge(optional_params) }
  let(:resource) { described_class.new(params) }
  let(:provider) { Puppet::Provider.new(resource) }
  let(:catalog) { Puppet::Resource::Catalog.new }
  let(:error_pattern) { %r(failed on) }

  it "has :name as its keyattribute" do
    expect( described_class.key_attributes).to eq([:name])
  end

  describe "has property" do
    [
      # list properties
        :ensure,
        :cos,
        :maxbw,
        :priority,
        :protection,
        :ipaddr,
        :macaddr,
        :uuid,
    ].each { |prop|
      it prop do
        expect(described_class.attrtype(prop)).to eq(:property)
      end
    }
  end
  describe "has parameter" do
    [
        :name,
    ].each { |param|
      it param do
        expect(described_class.attrtype(param)).to eq(:param)
      end
    }
  end

# Input Validation
describe "parameter" do
  [:name].each do |type|
    context "#{type}" do
      context "accepts" do
        %w(foo/bar/baz).each do |thing|
          it thing.inspect do
            params[type] = thing
            expect { resource }.not_to raise_error
          end
        end
      end
      context "rejects" do
        %w(foo foo/bar/baz/quux).each do |thing|
          it thing.inspect do
            params[type] = thing
            expect { resource }.to raise_error(Puppet::Error, error_pattern)
          end
        end
      end
      it "fails if unset" do
        params.delete(type)
        expect { resource }.to raise_error(Puppet::Error, %r(must be provided))
      end
    end
  end
end # parameters

describe "property" do
    [:ensure].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(present absent).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(true false).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:cos].each do |type|
      context "#{type}" do
        context "accepts" do
          (0..7).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(yes 8 -1).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:maxbw].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(1 1m 1M 1k 1K 1g 1G).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(1T 1.1 string).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:priority].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(low medium high).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(slow fast).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        it "defaults to medium" do
          expect(resource[type]).to eq(:medium)
        end
      end
    end
    [:protection].each do |type|
      context "#{type}" do
        context "accepts" do
          # Build all the combinations
          (
            (1..4).flat_map{|size|
            %w(mac-nospoof restricted ip-nospoof dhcp-nospoof).
              combination(size).to_a } + [["none"]]
          ).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          (%w(other) + [[ "mac-nospoof", "none"]]).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error,
                %r(Validation.*failed|Invalid value))
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        it "defaults to mac-nospoof,ip-nospoof" do
          expect(resource[type]).to eq([:"mac-nospoof", :"ip-nospoof"])
        end
      end
    end
    [:ipaddr].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(1.2.3.4).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(1.2.3.456).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
    [:macaddr].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(12:34:56:78:9a:bc).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(p2:34:56:78:9a:bc).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
        ## Test any munge behavior for parameter here
        #it "munges" do
        #  params[type] = %w(random_string)
        #  expect(resource[type]).to eq('other_string')
        #end
      end
    end
    [:uuid].each do |type|
      context "#{type}" do
        context "accepts" do
          %w(6c982de8-df63-4d1b-b3af-441cf2eb1959).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.not_to raise_error
            end
          end
        end
        context "rejects" do
          %w(Pc982de8-df63-4d1b-b3af-441cf2eb1959).each do |thing|
            it thing.inspect do
              params[type] = thing
              expect { resource }.to raise_error(Puppet::Error, error_pattern)
            end
          end
        end
        it "does not fail if unset" do
          params.delete(type)
          expect { resource }.not_to raise_error
        end
      end
    end
end # properties


# describe "autorequire" do
#   # This assumes resource requires thing => two
#   it "does not require thing when no matching resource exists" do
#     thing = Puppet::Type.type(:thing).new(:name => "one")
#     catalog.add_resource thing
#     catalog.add_resource resource
#     expect(resource.autorequire.count).to eq 0
#   end
#   it "requires the matching thing resource" do
#     thing = Puppet::Type.type(:thing).new(:name => "two")
#     catalog.add_resource thing
#     catalog.add_resource resource
#     reqs = resource.autorequire
#     expect(reqs.count).to eq 1
#     expect(reqs[0].source).to eq thing
#     expect(reqs[0].target).to eq resource
#   end
# end
end
